function TheCost = PlantCost(Y, U, a, b, ig, s, Transient, DC, Nyquist);
%
%	TheCost = PlantCost(Y, U, a, b, ig, s, Transient, DC, Nyquist); 
%
% Calculates the cost function for identifying plant models with arb. signals
%
% output arguments
%   TheCost = value cost function
%
% input arguments
%   Y			=	DFT spectrum output signal
%   U			=	DFT spectrum input signal
%	a			=	coefficients numerator noise model, dimensions: OrderC+1 x 1
%	b			=	coefficients denominator noise model, dimensions: OrderD+1 x 1
%	ig			=	coefficients numerator transient polynomial T, dimensions: OrderT1+1 x 1
%	s			=	vector of (jwk) or (zk^-1) values, dimension: number of frequencies x 1
%	Transient	=	if Transient = 1 then transient polynomial T is estimated
%	DC			=	1 if DC present otherwise 0
%	Nyquist		=	1 if Nyquist frequency present otherwise 0
%

A = polyval(fliplr(a),s);
B = polyval(fliplr(b),s);
if Transient == 1
	Ig = polyval(fliplr(ig),s);
else
	Ig = [];
end;
Error = PlantError(Y, U, A, B, Ig, Transient, DC, Nyquist);
TheCost = real(Error'*Error);
