function TheError = PlantError(Y, U, A, B, Ig, Transient, DC, Nyquist);
%
%	TheError = PlantError(Y, U, A, B, Ig, Transient, DC, Nyquist)
%
%	output parameters
%		TheError	=	complex error model equations
%
%	input parameters
%   Y			=	DFT spectrum output signal
%   U			=	DFT spectrum input signal
%	A			=	denominator polynomial plant model evaluated at the DFT freq.
%	B			=	numerator polynomial plant model evaluated at the DFT freq.
%	Ig			=	initial condition polynomial evaluated at the DFT freq.
%	Transient	=	if Transient = 1 then transient polynomial T is estimated
%	DC			=	1 if DC present otherwise 0
%	Nyquist		=	1 if Nyquist frequency present otherwise 0
%

TheError = (A.*Y-B.*U);
if Transient == 1
	TheError = TheError - Ig;
end;
if DC == 1 TheError(1) = TheError(1)/sqrt(2); end
if Nyquist == 1 TheError(length(TheError)) = TheError(length(TheError))/sqrt(2); end

TheError = TheError(:);
