function TheData = DataStruct(Y, U, freq, Ts, Gc);
%
% TheData = DataStruct(Y, U, freq, Ts, Gc);
%
%	Output parameter
%
%		TheData			=	structure containing the non-parametric data
%								TheData.Y		=	DFT spectrum ny x 1 output signal, dimensions ny x number of frequencies
%								TheData.U		=	DFT spectrum nu x 1 input signal, dimensions: nu x number of frequencies
%								TheData.freq	=	vector of frequency values (Hz), dimension: number of frequencies x 1
%								TheData.Ts		=	sampling time (s)
%								TheData.Gc		=	controller transfer function, zero or empty if unknown or not present,
%													dimension nu x ny x number of frequencies
%
%	Input parameters
%
%							Y		=	DFT spectrum ny x 1 output signal, dimensions ny x number of frequencies
%							U		=	DFT spectrum nu x 1 input signal, dimensions: nu x number of frequencies
%							freq	=	vector of frequency values (Hz), dimension: number of frequencies x 1
%							Ts		=	sampling time (s)
%							Gc		=	controller transfer function, zero or empty if unknown or not present,
%										dimension nu x ny x number of frequencies
%
%
% Copyright (c) Rik Pintelon, Vrije Universiteit Brussel - dept. ELEC, December 2004 
% All rights reserved.
% Software can be used freely for non-commercial applications only.
%

TheData = struct('Y',[],'U',[],'freq',[],'Ts',[],'Gc',[]);
TheData.Y = Y;
TheData.U = U;
TheData.freq = freq;
TheData.Ts = Ts;
TheData.Gc = Gc;
