function TheMat = MIMO_ML_SelectEstimParam(TheMat, TheSel);
%
% function TheMat = MIMO_ML_SelectEstimParam(TheMat, TheSel);
%
%   selects the columns of the matrix TheMat corresponding to the estimated parameters
%
%
%	Output parameter
%
%		TheMat			    =	matrix where the columns contain only the estimated model parameters
%								size: rows x number of free parameters
%
%
%	Input parameter
%
%		TheMat		    	=	matrix where the columns contain all model parameters
%								size: rows x number of all parameters
%		TheSel				=	structure with fields 'A', 'B', 'Ig'
%									TheSel = struct('A',[],'B',[], 'Ig', [])
%									TheSel.A = 1 x (OrderA+1)
%										TheSel.A(r) = 1 if coeff. a(r-1) is unknown
%										Sel.A(r) = 0 if coeff. a(r-1) = 0
%									TheSel.B = ny x nu x (OrderB+1)
%										TheSel.B(i,j,r) = 1 if coeff. b(i,j,r-1) is unknown
%										TheSel.B(i,j,r) = 0 if coeff. b(i,j,r-1) = 0
%									TheSel.Ig = ny x (OrderIg+1)
%										TheSel.Ig(i,r) = 1 if coeff. ig(i,r-1) is unknown
%										TheSel.Ig(i,r) = 0 if coeff. ig(i,r-1) = 0
%
%
% Copyright (c) Rik Pintelon, Vrije Universiteit Brussel - dept. ELEC, November 2009
% All rights reserved.
% Software can be used freely for non-commercial applications only.
%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% create row vector that selects the columns of the %
% matrix corresponding to the estimated parameters  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% A polynomial
Select = TheSel.A;

% B ny x nu matrix polynomial
% for jj = 1:nu
% 	for ii = 1:ny
% 		Select = [Select, squeeze(TheSel.B(ii,jj,:)).'];
% 	end % ii
% end % jj
SelB = permute(TheSel.B,[3,1,2]);
Select = [Select, SelB(:).'];

% Ig ny x 1 vector polynomial
% for ii = 1:ny
% 	Select = [Select, TheSel.Ig(ii,:)];
% end % ii
SelIg = TheSel.Ig.';
Select = [Select, SelIg(:).'];


%%%%%%%%%%%%%%%%%%%%%%%%%%%
% selection columns Jacob %
%%%%%%%%%%%%%%%%%%%%%%%%%%%

TheMat = TheMat(:, Select == 1);
