% Test for the removal of shifted transients in
%
%                        FRF measurements
%
% to allow to concatenated multiple data records
%
% Transient is estimated by adding additional input at Transient Time
%
% In this version you can concatenate records of different lenghts
% 9 November 2009, a gray day
%
% Copyright: 
% Johan Schoukens and Rik Pintelon 
% Vrije Universiteit Brussels, Pleinlaan 2, 1050 Brussels, Belgium




clear all
clf
% define the input variables for rectangular window, fs normalized to 1

NAll=[1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1]*4096;   % length of the individual data blocks
NPlot=[1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16];
SNR=0       % set noise power relative to output power
Ampl=1         % RMS-value of the excitation
NTrans=1024        % to eliminate transient effects in the simulation

% system + generator filter
[b,a]=cheby1(2,20,0.2);b=b*10;b(3)=b(3)+0.01;
[bGen,aGen]=butter(2,0.4*2);   % generator filter

NAllTrans=[0 NAll];    % intermediate variable

for k=1:length(NAll)  % run of the successive data blocks
    disp('number of concatenated experiments'),disp(k)

N=NAll(k);     % total length of the actual record

Ntotal=sum(NAll(1:k)); % total length of the concatenated record
Lines=[1:NPlot(k):Ntotal/2-2];% select the lines to be plotted
f{k}=[Lines-1]'/Ntotal; % frequency vector


% generate excitation signals
    
xIn=randn(1,N+NTrans);xIn=filter(bGen,aGen,xIn);xIn=xIn/std(xIn)*Ampl;
xOut=filter(b,a,xIn);      % calculate output
xOut=xOut+sqrt(SNR*var(xOut))*randn(size(xOut));   % add disturbing noise
xIn(1:NTrans)=[];xOut(1:NTrans)=[];   % eliminate transient
xInAll{k}=xIn;xOutAll{k}=xOut;      % store the signals

    
% create the concatinated signals
u=[xInAll{1:k}];y=[xOutAll{1:k}];

% processing with Local Polynomial method
U=fft(u)/sqrt(Ntotal);Y=fft(y)/sqrt(Ntotal);U(Ntotal/2-1:end)=[];Y(Ntotal/2-1:end)=[];

DataPoly.Y = Y(:).';   % store spectra as rows

% Estimate the transient by adding additional input of exp(-j*tau*w)

% settings of the parameters LocalPolynomial method
method.order = 2;    % degree local polynoom
method.transient=0;  % transient is estimated via the inputs
method.step=k;
% create transient inputs
clear UTrans 
for r=1:k
  x=zeros(1,Ntotal);
  nTransStart=sum(NAllTrans(1:r))+1;
  x(nTransStart)=1;
  X=fft(x);X(Ntotal/2-1:end)=[];
  UTrans(r,:)=X;
end   % end loop to create transient inputs
                    
DataPoly.U = [U(:).'; UTrans];  % add ones for the transient estimate

[CY, Ym, TY, Gest, CvecG2OE,M] = LocalPolyAnal(DataPoly, method);  % estimate G, ...
   
Gest = Gest(1,1,:);
%Gest=Gest(Lines);
G{k}=Gest(:);   % take out the FRF
G0{k}=freqz(b,a,f{k}*2*pi);
E{k}=G{k}-G0{k};

subplot(4,4,k)
plot(f{k},db(G0{k}),f{k},db(E{k}),'.'),shg

end  % loop over the data blocks

ColVec=['k','b','r','g','k','b'];
figure(3),clf,hold on
for k=1:6
    [k (db(max(E{k}))) (db(var(E{k}))/2) (length(E{k}))]
    stdE(k)=((std(E{k})));
    plot(f{k},db(G0{k}),ColVec(k),f{k},db(E{k}),ColVec(k)),shg
end
hold off
        
