clear all
clf
Nrep=1000;   % repeat the whole process Nrep times

for nRep=1:Nrep      % loop over repeated simulations of the whole process
 % define the input variables for rectangular window, fs normalized to 1   
Nsub=4096;   % length of the individual sub records
MsubRecord=8;   % number of sub records
SNR=0.1      % set noise power relative to output power
Ampl=1         % RMS-value of the excitation
NTrans=1024        % to eliminate transient effects in the simulation

method.moment=20      % --> set degrees of freedom such that 2nd moment inverse variance estim exist




% system + generator filter
[b,a]=cheby1(2,20,0.2);b=b*10;b(3)=b(3)+0.01;
[bGen,aGen]=butter(2,0.4*2);   % generator filter


% generate the excitation signals
for k=MsubRecord:-1:1    % run over the M sub records
    xIn=randn(1,Nsub+NTrans);xIn=filter(bGen,aGen,xIn);xIn=xIn/std(xIn)*Ampl;
    xOut=filter(b,a,xIn);      % calculate output
    xOut=xOut+sqrt(SNR*var(xOut))*randn(size(xOut));   % add disturbing noise
    xIn(1:NTrans)=[];xOut(1:NTrans)=[];   % eliminate transient
    xInAll{k}=xIn(:)';xOutAll{k}=xOut(:)';      % store the signals
end

% processing the individual subrecord
N=Nsub;
Lines=[1:N/2-2];% select the lines to be plotted
f=[Lines-1]'/N; % frequency vector
G0=freqz(b,a,f*2*pi);    % exact system


for k=1:MsubRecord  % run of the successive data blocks
 %disp('sub record'),k 
% create the signals
u=[xInAll{k}];y=[xOutAll{k}];

% processing with Local Polynomial method
U=fft(u)/sqrt(N);Y=fft(y)/sqrt(N);U(N/2-1:end)=[];Y(N/2-1:end)=[];

DataPoly.Y = Y(:).';   % store spectra as rows

% Estimate the transient by adding additional input of exp(-j*tau*w)

% settings of the parameters LocalPolynomial method
method.order = 2;    % degree local polynoom
method.transient=0;  % transient is estimated via the inputs
method.step=1;       % freq. step


% create transient inputs
  clear UTrans
  x=zeros(1,N);
  nTransStart=1;
  x(nTransStart)=1;
  X=fft(x);X(N/2-1:end)=[];
  UTrans(1,:)=X;
                    
DataPoly.U = [U(:).'; UTrans];  % add ones for the transient estimate

[CY, Ym, TY, Gest, CvecG2OE,M] = LocalPolyAnal(DataPoly, method);  % estimate G, ...
   
Gest = Gest(1,1,:);Gest=Gest(Lines);G1All(k,:)=Gest(:).';   % take out the FRF

end   % end loop over the individual sub records

G1varAll(:,nRep)= squeeze(CvecG2OE(1,1,:));   % take out the cov. matrices

G1=mean(G1All);   % average over the M sub-blocks
E=G1(:)-G0;

G1mAll(nRep,:)=G1;


%
% process the data with all sub records concatenated
%

N=Nsub*MsubRecord;     % total data length
Lines=[1:MsubRecord:N/2-2];% select the lines to be plotted
f2=[Lines-1]'/N; % frequency vector
G02=freqz(b,a,f2*2*pi);    % exact system

u=[xInAll{1:MsubRecord}];y=[xOutAll{1:MsubRecord}];

% processing with Local Polynomial method
U=fft(u)/sqrt(N);Y=fft(y)/sqrt(N);U(N/2-1:end)=[];Y(N/2-1:end)=[];

DataPoly.Y = Y(:).';   % store spectra as rows

% end loop to create transient inputs
                    % create transient inputs
clear UTrans 
for r=1:MsubRecord
    
  x=zeros(1,N);
  nTransStart=1+(r-1)*Nsub;
  x(nTransStart)=1;
  X=fft(x);X(N/2-1:end)=[];
  UTrans(r,:)=X;
end   
DataPoly.U = [U(:).'; UTrans];  % add ones for the transient estimate
method.step=MsubRecord;

[CY, Ym, TY, Gest, CvecG2OE,M] = LocalPolyAnal(DataPoly, method);  % estimate G, ...

Gest = Gest(1,1,:);G2=Gest(:).';   % take out the FRF
G2All(nRep,:)=G2;

G2varAll(:,nRep)= squeeze(CvecG2OE(1,1,:));   % take out the cov. matrices


E2All(nRep,:)=G2(:)-G02;

%subplot(4,4,nRep)
%plot(f,db(G0),f,db(E),'.r',f2,db(G02),f2,db(E2),'.b'),shg

end   % loop over repeated simulation

%save RunM5_1000Noise


G1m=mean(G1mAll);
G2m=mean(G2All);G2m(end-1:end)=[];

G1std=std(G1mAll,0);
G2std=std(G2All,0);G2std(end-1:end)=[];

plot(f(:),db(G0(:)),f,db(G1std(:)),'.b',f,median(db(G1varAll(:,:)),2)/2,'b',...
                    f,db(G2std(:)),'.r',f,median(db(G2varAll(1:end-2,:)),2)/2,'r'),shg
                DG_SetTraceWidth(0.1,'*')
% plot(f(:),db(G0(:)),f,db(G0(:)-G1m(:)),'b',f,db(G1std(:)),'.b',...
%                     f,db(G0(:)-G2m(:)),'r',f,db(G2std(:)),'.r',f,median(db(GvarAll(1:end-1,:)),2)/2,'k'),shg
%                 DG_SetTraceWidth(0.1,'*')
xlabel('Input')
%DG_Init4PDF(gcf, 6); 
%DG_MakePDF('FigISMAratioBiasAndVarianceNoise.pdf', gcf)