% Read me first file: use of the function for detecting the nonlinear distortions in FRF measurements of SISO systems 
% Examples illustrating the use of the functions can be found in the folder "examples". To run these examples one
% also needs the "DesignMultisineExcitations" toolbox.
%
%%   1. FAST DETECTION, QUANTIFICATION, AND CLASSIFICATION OF THE NONLINEAR DISTORTIONS  
%
%           [Y, Yc, U, G, freq] = Fast_NL_Anal(Yall, Uall, ExcitedHarm, MeasHarm, fs, N);
%
%       Starting from noisy input and noisy output observations (P > 1 periods) of the steady state response of a SISO system to 
%       one or more (M >= 1) full or odd random phase multisine excitations with random harmonic grid, the following quantities 
%       are calculated: 
%
%           1. The FRF, its noise variance, its total variance (noise + NL distortions), and the level of the NL distortions 
%              (full and odd multisines).
%           2. The mean input-output DFT spectra, the noise variance, the total variance, and the level of the NL distortions 
%              (full and odd multisines).
%           3. Classification in odd and even distortions
%              (odd multisines only). 
%
%       The results can be plotted using the function
%
%           Plot_Fast_NL_Anal(Y, Yc, U, G, freq, Spacing, RealNum, FigNum) 
%
%       Notes: 
%           1. The full an odd random phase multisine with random harmonic grid are generated by the "DesignMultisineExcitations" toolbox. 
%           2. If M >1, then also the robust estimate is calculated and compared with the fast estimate. 
%           3. The information of the NL distortions at the excited harmonics is obtained via extrapolation of the level of the 
%              NL distortions at the non-excited harmonics.  
%
%       References:
%
%           Pintelon R., G. Vandersteen, L. De Locht, Y. Rolain and J. Schoukens (2004). Experimental characterization of 
%           operational amplifiers: a system identification approach - Part I: theory and simulations, IEEE Trans. Instrum. Meas., 
%           vol. 53, no. 3, pp. 854-862.
%
%           Schoukens, J., J. Lataire, R. Pintelon, G. Vandersteen, and T. Dobrowiecki (2009). Robustness issues of the equivalent 
%           linear representation of a nonlinear system, IEEE Trans. Instrum. and Meas., vol. 58, no. 5, pp. 1737-1745.
%
%           Pintelon, R., and J. Schoukens (2012). System Identification: A Frequency Domain Approach, second edition, 
%           IEEE Press-Wiley, Piscataway (USA). 
%
%
%%   2. ROBUST DETECTION AND QUANTIFICATION OF THE NONLINEAR DISTORTIONS
%
%           [G, Y, U, CYU] = Robust_NL_Anal(Yall, Uall, Rall); 
%
%       Detects and quantifies the nonlinear distortions starting from P > 1 periods of the steady state response to 
%       M > 1 independent random phase multisine excitations. The analysis is performed at the excited frequency lines only. 
%       The following quantities are calculated: 
%
%           1. the FRF, its noise variance, its total variance (noise + NL distortions), and the level of the NL distortions 
%           2. the mean input-output DFT spectra, the noise variance, the total variance, and the level of the NL distortions 
%
%       The results can be plotted using the function
%
%           Plot_Robust_NL_Anal(G, Y, U, freq, Spacing, FigNum) 
%
%       Note: 
%           The NL distortions are quantified without any extrapolation via direct application of the definition of the best
%           linear approximation.
%
%       References:
%
%           Pintelon R. and J. Schoukens (2002). Measurement and modeling of linear systems in the presence of non-linear 
%           distortions. Mechanical Systems and Signal Processing, vol. 16, no. 5, pp. 785-801.
%
%           Pintelon R., G. Vandersteen, L. De Locht, Y. Rolain and J. Schoukens (2004). Experimental characterization of 
%           operational amplifiers: a system identification approach - Part I: theory and simulations, IEEE Trans. Instrum. Meas., 
%           vol. 53, no. 3, pp. 854-862.
%
%           Schoukens, J., J. Lataire, R. Pintelon, G. Vandersteen, and T. Dobrowiecki (2009). Robustness issues of the equivalent 
%           linear representation of a nonlinear system, IEEE Trans. Instrum. and Meas., vol. 58, no. 5, pp. 1737-1745.
%
%           Pintelon, R., and J. Schoukens (2012). System Identification: A Frequency Domain Approach, second edition, 
%
%
%%
%
% Copyright (c) Rik Pintelon, Vrije Universiteit Brussel - dept. ELEC, 12 October 2011 
% All rights reserved.
% Software can be used freely for non-commercial applications only.
%


