function Gest = sysid_multiple_traj(uu, yy, n)

addpath ~/mfiles/FrequencyDomainToolbox/LocalPolynomialMethod/auxiliary/

% define the input variables for rectangular window, fs normalized to 1   
MsubRecord=8;   % number of sub records
NTrans=1024;        % to eliminate transient effects in the simulation
method.moment=20;      % --> set degrees of freedom such that 2nd moment inverse variance estim exist

K = length(yy);
for k = 1:K, NN(k) = length(yy{k}); end

% processing the individual subrecord
for k=1:K  
    u = uu{k}; y = yy{k}; N = NN(k);
    U=fft(u)/sqrt(N);Y=fft(y)/sqrt(N);U(N/2-1:end)=[];Y(N/2-1:end)=[]; DataPoly.Y = Y(:).';   

    Lines = [1:N/2-2]; % select the lines to be plotted
    f=[Lines-1]'/N; % frequency vector

    % Estimate the transient by adding additional input of exp(-j*tau*w) settings of the parameters LocalPolynomial method
    method.order     = 2;    % degree local polynoom
    method.transient = 0;  % transient is estimated via the inputs
    method.step      = 1;       % freq. step
    
    % create transient inputs
    x=zeros(1,N);
    nTransStart=1;
    x(nTransStart)=1;
    X=fft(x);X(N/2-1:end)=[];
    UTrans(1,:)=X;
    
    DataPoly.U = [U(:).'; UTrans];  % add ones for the transient estimate
    [CY, Ym, TY, Gest, CvecG2OE, M] = LocalPolyAnal(DataPoly, method);
    Gest = Gest(1,1,:); Gest=Gest(Lines); G1All(k,:)=Gest(:).';   % take out the FRF
end

G1varAll= squeeze(CvecG2OE(1,1,:));   % take out the cov. matrices
G1=mean(G1All);   % average over the sub-blocks
G1mAll=G1;

% process the data with all sub records concatenated
N = sum(NN);       % total data length
Lines=[1:K:N/2-2]; % select the lines to be plotted
f2=[Lines-1]'/N;   % frequency vector
u=[uu{1:K}];y=[yy{1:K}];

% processing with Local Polynomial method
U=fft(u)/sqrt(N);Y=fft(y)/sqrt(N);U(N/2-1:end)=[];Y(N/2-1:end)=[];
DataPoly.Y = Y(:).';   % store spectra as rows

clear UTrans 
nTransStart=1;
for r=1:K    
  x=zeros(1,N);
  x(nTransStart)=1;
  X=fft(x);X(N/2-1:end)=[];
  UTrans(r,:)=X;
  nTransStart=nTransStart+NN(r);
end   
DataPoly.U = [U(:).'; UTrans];  % add ones for the transient estimate
method.step=K;

[CY, Ym, TY, Gest, CvecG2OE,M] = LocalPolyAnal(DataPoly, method);  % estimate G, ...
Gest = Gest(1,1,:); G2=Gest(:).';   % take out the FRF
G2All=G2;
G2varAll= squeeze(CvecG2OE(1,1,:));   % take out the cov. matrices
G1m=mean(G1mAll);
G2m=mean(G2All); % G2m(end-1:end)=[];
G1std=std(G1mAll,0);
G2std=std(G2All,0); % G2std(end-1:end)=[];
